@@ -1,12 +1,9 @@
-"""
-
-"""
-from tensorflow import keras
+from tensorflow.keras import activations
 from tensorflow.keras import backend as K
-from tensorflow.keras.layers import Layer
+from tensorflow.keras import constraints, initializers, layers, regularizers
 
 
-class ScaledDotProductAttention(Layer):
+class ScaledDotProductAttention(layers.Layer):
     """
     注意力模型主要包含三个输入 即Q(queries), K(keys), V(values)。按照下面这个公式计算的
 
@@ -116,7 +113,7 @@ class ScaledDotProductAttention(Layer):
         return v
 
 
-class SeqSelfAttention(Layer):
+class SeqSelfAttention(layers.Layer):
     ATTENTION_TYPE_ADD = 'additive'
     ATTENTION_TYPE_MUL = 'multiplicative'
 
@@ -171,15 +168,14 @@ class SeqSelfAttention(Layer):
 
         self.use_additive_bias = use_additive_bias
         self.use_attention_bias = use_attention_bias
-        self.kernel_initializer = keras.initializers.get(kernel_initializer)
-        self.bias_initializer = keras.initializers.get(bias_initializer)
-        self.kernel_regularizer = keras.regularizers.get(kernel_regularizer)
-        self.bias_regularizer = keras.regularizers.get(bias_regularizer)
-        self.kernel_constraint = keras.constraints.get(kernel_constraint)
-        self.bias_constraint = keras.constraints.get(bias_constraint)
-        self.attention_activation = keras.activations.get(attention_activation)
+        self.kernel_initializer = initializers.get(kernel_initializer)
+        self.bias_initializer = initializers.get(bias_initializer)
+        self.kernel_regularizer = regularizers.get(kernel_regularizer)
+        self.bias_regularizer = regularizers.get(bias_regularizer)
+        self.kernel_constraint = constraints.get(kernel_constraint)
+        self.bias_constraint = constraints.get(bias_constraint)
+        self.attention_activation = activations.get(attention_activation)
         self.attention_regularizer_weight = attention_regularizer_weight
-        self._backend = keras.backend.backend()
 
         if attention_type == SeqSelfAttention.ATTENTION_TYPE_ADD:
             self.Wx, self.Wt, self.bh = None, None, None
@@ -199,13 +195,13 @@ class SeqSelfAttention(Layer):
             'history_only': self.history_only,
             'use_additive_bias': self.use_additive_bias,
             'use_attention_bias': self.use_attention_bias,
-            'kernel_initializer': keras.regularizers.serialize(self.kernel_initializer),
-            'bias_initializer': keras.regularizers.serialize(self.bias_initializer),
-            'kernel_regularizer': keras.regularizers.serialize(self.kernel_regularizer),
-            'bias_regularizer': keras.regularizers.serialize(self.bias_regularizer),
-            'kernel_constraint': keras.constraints.serialize(self.kernel_constraint),
-            'bias_constraint': keras.constraints.serialize(self.bias_constraint),
-            'attention_activation': keras.activations.serialize(self.attention_activation),
+            'kernel_initializer': regularizers.serialize(self.kernel_initializer),
+            'bias_initializer': regularizers.serialize(self.bias_initializer),
+            'kernel_regularizer': regularizers.serialize(self.kernel_regularizer),
+            'bias_regularizer': regularizers.serialize(self.bias_regularizer),
+            'kernel_constraint': constraints.serialize(self.kernel_constraint),
+            'bias_constraint': constraints.serialize(self.bias_constraint),
+            'attention_activation': activations.serialize(self.attention_activation),
             'attention_regularizer_weight': self.attention_regularizer_weight,
         }
         base_config = super(SeqSelfAttention, self).get_config()
@@ -254,13 +250,13 @@ class SeqSelfAttention(Layer):
         feature_dim = int(input_shape[2])
 
         self.Wa = self.add_weight(shape=(feature_dim, feature_dim),
-                                  name='{}_Mul_Wa'.format(self.name),
+                                  name='kernel'.format(self.name),
                                   initializer=self.kernel_initializer,
                                   regularizer=self.kernel_regularizer,
                                   constraint=self.kernel_constraint)
         if self.use_attention_bias:
             self.ba = self.add_weight(shape=(1,),
-                                      name='{}_Mul_ba'.format(self.name),
+                                      name='bias'.format(self.name),
                                       initializer=self.bias_initializer,
                                       regularizer=self.bias_regularizer,
                                       constraint=self.bias_constraint)
@@ -319,17 +315,14 @@ class SeqSelfAttention(Layer):
 
         # e_{t, t'} = W_a h_{t, t'} + b_a
         if self.use_attention_bias:
-            e = K.reshape(K.dot(h, self.Wa) + self.ba,
-                          (batch_size, input_len, input_len))
+            e = K.reshape(K.dot(h, self.Wa) + self.ba, (batch_size, input_len, input_len))
         else:
-            e = K.reshape(K.dot(h, self.Wa),
-                          (batch_size, input_len, input_len))
+            e = K.reshape(K.dot(h, self.Wa), (batch_size, input_len, input_len))
         return e
 
     def _call_multiplicative_emission(self, inputs):
         # e_{t, t'} = x_t^T W_a x_{t'} + b_a
-        e = K.batch_dot(K.dot(inputs, self.Wa),
-                        K.permute_dimensions(inputs, (0, 2, 1)))
+        e = K.batch_dot(K.dot(inputs, self.Wa), K.permute_dimensions(inputs, (0, 2, 1)))
         if self.use_attention_bias:
             e += self.ba[0]
         return e
@@ -361,7 +354,7 @@ class SeqSelfAttention(Layer):
         return {'SeqSelfAttention': SeqSelfAttention}
 
 
-class SeqWeightedAttention(Layer):
+class SeqWeightedAttention(layers.Layer):
     r"""Y = \text{softmax}(XW + b) X
 
     See: https://arxiv.org/pdf/1708.00524.pdf
@@ -386,11 +379,11 @@ class SeqWeightedAttention(Layer):
     def build(self, input_shape):
         self.W = self.add_weight(shape=(int(input_shape[2]), 1),
                                  name='{}_W'.format(self.name),
-                                 initializer=keras.initializers.get('uniform'))
+                                 initializer=initializers.get('uniform'))
         if self.use_bias:
             self.b = self.add_weight(shape=(1,),
                                      name='{}_b'.format(self.name),
-                                     initializer=keras.initializers.get('zeros'))
+                                     initializer=initializers.get('zeros'))
         super(SeqWeightedAttention, self).build(input_shape)
 
     def call(self, x, mask=None):
@@ -426,7 +419,7 @@ class SeqWeightedAttention(Layer):
         return {'SeqWeightedAttention': SeqWeightedAttention}
 
 
-class MultiHeadAttention(Layer):
+class MultiHeadAttention(layers.Layer):
     """Multi-head attention layer.
 
     See: https://arxiv.org/pdf/1706.03762.pdf
@@ -459,14 +452,14 @@ class MultiHeadAttention(Layer):
         """
         self.supports_masking = True
         self.head_num = head_num
-        self.activation = keras.activations.get(activation)
+        self.activation = activations.get(activation)
         self.use_bias = use_bias
-        self.kernel_initializer = keras.initializers.get(kernel_initializer)
-        self.bias_initializer = keras.initializers.get(bias_initializer)
-        self.kernel_regularizer = keras.regularizers.get(kernel_regularizer)
-        self.bias_regularizer = keras.regularizers.get(bias_regularizer)
-        self.kernel_constraint = keras.constraints.get(kernel_constraint)
-        self.bias_constraint = keras.constraints.get(bias_constraint)
+        self.kernel_initializer = initializers.get(kernel_initializer)
+        self.bias_initializer = initializers.get(bias_initializer)
+        self.kernel_regularizer = regularizers.get(kernel_regularizer)
+        self.bias_regularizer = regularizers.get(bias_regularizer)
+        self.kernel_constraint = constraints.get(kernel_constraint)
+        self.bias_constraint = constraints.get(bias_constraint)
         self.history_only = history_only
 
         self.Wq, self.Wk, self.Wv, self.Wo = None, None, None, None
@@ -476,14 +469,14 @@ class MultiHeadAttention(Layer):
     def get_config(self):
         config = {
             'head_num': self.head_num,
-            'activation': keras.activations.serialize(self.activation),
+            'activation': activations.serialize(self.activation),
             'use_bias': self.use_bias,
-            'kernel_initializer': keras.initializers.serialize(self.kernel_initializer),
-            'bias_initializer': keras.initializers.serialize(self.bias_initializer),
-            'kernel_regularizer': keras.regularizers.serialize(self.kernel_regularizer),
-            'bias_regularizer': keras.regularizers.serialize(self.bias_regularizer),
-            'kernel_constraint': keras.constraints.serialize(self.kernel_constraint),
-            'bias_constraint': keras.constraints.serialize(self.bias_constraint),
+            'kernel_initializer': initializers.serialize(self.kernel_initializer),
+            'bias_initializer': initializers.serialize(self.bias_initializer),
+            'kernel_regularizer': regularizers.serialize(self.kernel_regularizer),
+            'bias_regularizer': regularizers.serialize(self.bias_regularizer),
+            'kernel_constraint': constraints.serialize(self.kernel_constraint),
+            'bias_constraint': constraints.serialize(self.bias_constraint),
             'history_only': self.history_only,
         }
         base_config = super(MultiHeadAttention, self).get_config()
@@ -514,53 +507,53 @@ class MultiHeadAttention(Layer):
                                   initializer=self.kernel_initializer,
                                   regularizer=self.kernel_regularizer,
                                   constraint=self.kernel_constraint,
-                                  name='%s_Wq' % self.name,
+                                  name='query/kernel',
                                   )
         if self.use_bias:
             self.bq = self.add_weight(shape=(feature_dim,),
                                       initializer=self.bias_initializer,
                                       regularizer=self.bias_regularizer,
                                       constraint=self.bias_constraint,
-                                      name='%s_bq' % self.name,
+                                      name='query/bias',
                                       )
         self.Wk = self.add_weight(shape=(int(k[-1]), feature_dim),
                                   initializer=self.kernel_initializer,
                                   regularizer=self.kernel_regularizer,
                                   constraint=self.kernel_constraint,
-                                  name='%s_Wk' % self.name,
+                                  name='key/kernel',
                                   )
         if self.use_bias:
             self.bk = self.add_weight(shape=(feature_dim,),
                                       initializer=self.bias_initializer,
                                       regularizer=self.bias_regularizer,
                                       constraint=self.bias_constraint,
-                                      name='%s_bk' % self.name,
+                                      name='key/bias',
                                       )
         self.Wv = self.add_weight(shape=(int(v[-1]), feature_dim),
                                   initializer=self.kernel_initializer,
                                   regularizer=self.kernel_regularizer,
                                   constraint=self.kernel_constraint,
-                                  name='%s_Wv' % self.name,
+                                  name='value/kernel',
                                   )
         if self.use_bias:
             self.bv = self.add_weight(shape=(feature_dim,),
                                       initializer=self.bias_initializer,
                                       regularizer=self.bias_regularizer,
                                       constraint=self.bias_constraint,
-                                      name='%s_bv' % self.name,
+                                      name='value/bias',
                                       )
         self.Wo = self.add_weight(shape=(feature_dim, feature_dim),
                                   initializer=self.kernel_initializer,
                                   regularizer=self.kernel_regularizer,
                                   constraint=self.kernel_constraint,
-                                  name='%s_Wo' % self.name,
+                                  name='output/kernel',
                                   )
         if self.use_bias:
             self.bo = self.add_weight(shape=(feature_dim,),
                                       initializer=self.bias_initializer,
                                       regularizer=self.bias_regularizer,
                                       constraint=self.bias_constraint,
-                                      name='%s_bo' % self.name,
+                                      name='output/bias',
                                       )
         super(MultiHeadAttention, self).build(input_shape)
 
@@ -593,8 +586,8 @@ class MultiHeadAttention(Layer):
 
     def call(self, inputs, mask=None):
         """
-        :param inputs: 
-        :param mask: 
+        :param inputs:
+        :param mask:
         """
         if isinstance(inputs, list):
             q, k, v = inputs
@@ -615,23 +608,22 @@ class MultiHeadAttention(Layer):
             q = self.activation(q)
             k = self.activation(k)
             v = self.activation(v)
-        y = ScaledDotProductAttention(history_only=self.history_only,
-                                      name='%s-Attention' % self.name,
-                                      )(inputs=[self._reshape_to_batches(q, self.head_num),
-                                                self._reshape_to_batches(
-                                                    k, self.head_num),
-                                                self._reshape_to_batches(
-                                                    v, self.head_num),
-                                                ],
-                                        mask=[
-                                            self._reshape_mask(
-                                                q_mask, self.head_num),
-                                            self._reshape_mask(
-                                                k_mask, self.head_num),
-                                            self._reshape_mask(
-                                                v_mask, self.head_num),
-                                      ],
-        )
+        y = ScaledDotProductAttention(
+            history_only=self.history_only,
+            name='attention',
+        )(inputs=[self._reshape_to_batches(q, self.head_num),
+                  self._reshape_to_batches(k, self.head_num),
+                  self._reshape_to_batches(v, self.head_num),
+                  ],
+          mask=[
+              self._reshape_mask(
+                  q_mask, self.head_num),
+              self._reshape_mask(
+                  k_mask, self.head_num),
+              self._reshape_mask(
+                  v_mask, self.head_num),
+          ],
+          )
         y = self._reshape_from_batches(y, self.head_num)
         y = K.dot(y, self.Wo)
         if self.use_bias:
